"""Gateway CGI methods.
"""
#
# Copyright (c) 2005,2006 shinGETsu Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Id: gateway_cgi.py 1207 2006-06-04 14:10:05Z fuktommy $
#

import re
import cgi
import csv
from time import time

import config
import gateway
from cache import *
from rss import RSS, make_rss1, make_rss2

__version__ = "$Revision: 1207 $"


class CGI(gateway.CGI):

    """Class for /gateway.cgi."""

    def run(self):
        path = self.path_info()
        self.form = cgi.FieldStorage(environ=self.environ, fp=self.stdin)
        filter = self.form.getfirst('filter', '')
        if filter:
            try:
                self.filter = re.compile(unicode(filter, 'utf-8'), re.I)
            except (re.error, UnicodeDecodeError):
                self.header(self.message['regexp_error'], deny_robot=True)
                self.footer()
                return
            self.str_filter = cgi.escape(filter, True)

        if not self.check_visitor():
            self.print403()
            return
        elif path == "motd":
            self.print_motd()
        elif path == "rss":
            self.print_rss()
        elif path == "index":
            self.print_index()
        elif path == "changes":
            self.print_changes()
        elif path in ("recent", "new"):
            if (not self.isfriend) and (not self.isadmin):
                self.print403()
            elif path == "recent":
                self.print_recent()
            elif path == "new":
                self.header(self.message["new"], deny_robot=True)
                self.print_new_element_form()
                self.footer()
            else:
                self.print404()
        elif self.form.getfirst("cmd", "") == "new":
            self.jump_new_file()
        elif path.startswith("csv"):
            self.print_csv(path)
        elif re.search(r"^(thread)", path):
            m = re.search(r"^(thread)/?([^/]*)$", path)
            if m is None:
                self.print_title()
                return
            elif m.group(2) != "":
                uri = self.appli[m.group(1)] + self.sep + \
                      self.str_encode(m.group(2))
            elif self.environ.get("QUERY_STRING", "") != "":
                uri = self.appli[m.group(1)] + self.sep + \
                      self.environ["QUERY_STRING"]
            else:
                self.print_title()
                return

            self.print302(uri)
        elif path == '':
            self.print_title()
        else:
            self.print404()

    def print_title(self):
        message = self.message
        self.header(message["logo"] + " - " + message["description"])
        self.stdout.write('<ul class="topmenu">\n')
        for i in ("changes", "index"):
                self.stdout.write('  <li>%s</li>\n' %
                    self.gateway_link(self.gateway_cgi, i))
        if self.isfriend or self.isadmin:
            for i in ("recent", "new"):
                self.stdout.write('  <li>%s</li>\n' %
                    self.gateway_link(self.gateway_cgi, i))
        if self.isadmin:
            for i in ("search", "status"):
                self.stdout.write('  <li>%s</li>\n' %
                    self.gateway_link(self.admin_cgi, i))
        self.stdout.write(
            '  <li><a href="http://shingetsu.info/">' +
	    message["site"] + "</a></li>\n" +
            '  <li><a href="' + self.gateway_cgi + self.sep + 'motd">' +
            message["agreement"] + "</a></li>\n")
        self.stdout.write(
            '  <li><a href="%s%srss">%s</a></li>\n' %
            (self.gateway_cgi, self.sep, message["rss"]))
        self.stdout.write("</ul>\n")
        self.stdout.write("<h2>%s</h2>\n" % self.message['recent_changes'])
        self.stdout.write("<ul>\n")
        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        now = int(time())
        for cache in cachelist:
            if now <= cache.stamp + config.rss_range:
                self.stdout.write(self.make_list_item(cache, remove=False))
        self.stdout.write("</ul>\n")
        self.print_new_element_form()
        self.footer()

    def print_index(self):
        """Print index page."""
        if self.str_filter:
            title = '%s : %s' % (self.message['index'], self.str_filter)
        else:
            title = self.message['index']
        self.header(title)
        self.stdout.write('<p>%s</p>\n' % self.message['desc_index'])
        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(a.datfile, b.datfile))
        self.print_index_list(cachelist, "index")

    def print_changes(self):
        """Print changes page."""
        if self.str_filter:
            title = '%s : %s' % (self.message['changes'], self.str_filter)
        else:
            title = self.message['changes']
        self.header(title)
        self.stdout.write('<p>%s</p>\n' % self.message['desc_changes'])
        cachelist = CacheList()
        cachelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        self.print_index_list(cachelist, "changes")

    def make_recent_cachelist(self):
        """Make dummy cachelist from recentlist."""
        recentlist = RecentList()
        recentlist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        cachelist = []
        check = []
        for rec in recentlist:
            if rec.datfile not in check:
                cache = Cache(rec.datfile)
                cache.stamp = rec.stamp
                cachelist.append(cache)
                check.append(rec.datfile)
        return cachelist

    def print_recent(self):
        """Print changes page."""
        if self.str_filter:
            title = '%s : %s' % (self.message['recent'], self.str_filter)
        else:
            title = self.message['recent']
        self.header(title)
        self.stdout.write('<p>%s</p>\n' % self.message['desc_recent'])
        cachelist = self.make_recent_cachelist()
        self.print_index_list(cachelist, "recent")

    def print_csv(self, path):
        """CSV output as API."""
        found = re.search(r"^csv/([^/]+)/(.+)", path)
        if found:
            target, cols = found.groups()
        else:
            self.print404()
            return
        cols = cols.split(",")
        if target == "index":
            cachelist = CacheList()
        elif target == "changes":
            cachelist = CacheList()
            cachelist.sort(lambda a,b: cmp(b.stamp, a.stamp))
        elif target == "recent":
            if (not self.isfriend) and (not self.isadmin):
                self.print403()
                return
            cachelist = self.make_recent_cachelist()
        else:
            self.print404()
            return
        self.stdout.write("Content-Type: text/comma-separated-values;" +
                          " charset=UTF-8\n\n")
        writer = csv.writer(self.stdout)
        host = self.environ.get("HTTP_HOST", "")
        for cache in cachelist:
            title = self.file_decode(cache.datfile)
            if cache.type in config.types:
                type = cache.type
                path = self.appli[cache.type] + self.sep + \
                       self.str_encode(title)
            else:
                type = ""
                path = ""
            row = []
            for c in cols:
                if c == "file":
                    row.append(cache.datfile)
                elif c == "stamp":
                    row.append(cache.stamp)
                elif c == "date":
                    row.append(self.localtime(cache.stamp))
                elif c == "path":
                    row.append(path)
                elif c == "uri":
                    if host and path:
                        row.append("http://" + host + path)
                    else:
                        row.append("")
                elif c == "type":
                    row.append(cache.type)
                elif c == "title":
                    row.append(title)
                elif c == "records":
                    row.append(len(cache))
                elif c == "size":
                    row.append(cache.size)
                else:
                    row.append("")
            writer.writerow(row)

    def jump_new_file(self):
        if self.form.getfirst("link", "") == "":
            self.header(self.message["null_title"], deny_robot=True)
            self.footer()
        elif re.search(r"/|\]\]", self.form.getfirst("link", "")):
            self.header(self.message["bad_title"], deny_robot=True)
            self.footer()
        elif self.form.getfirst("type", "") == "":
            self.header(self.message["null_type"], deny_robot=True)
            self.footer()
        elif self.form.getfirst("type", "") in config.types:
            self.print302(self.appli[self.form.getfirst("type", "")] +
                          self.sep +
                          self.str_encode(self.form.getfirst("link", "")))
        else:
            self.print404()

    def rss_text_format(self, plain):
        buf = plain.replace("<br>", " ")
        buf = buf.replace("&", "&amp;")
        buf = re.sub(r'&amp;(#\d+|lt|gt|amp);', r'&\1;', buf)
        buf = buf.replace("<", "&lt;")
        buf = buf.replace(">", "&gt;")
        buf = buf.replace("\r", "")
        buf = buf.replace("\n", "")
        return buf

    def rss_html_format(self, plain):
        buf = self.escape(plain)
        buf = buf.replace('&lt;br&gt;', '<br>')
        buf = re.sub(r'<br> ', '<br>&nbsp;', buf)
        buf = re.sub(r'^ ', '&nbsp;', buf)
        buf = re.sub(r'<br>', '<br />\n    ',buf)
        buf = re.sub(r"https?://[^\x00-\x20\"'()<>\[\]\x7F-\xFF]{2,}",
                     r'<a href="\g<0>">\g<0></a>',
                     buf)
        if buf:
            buf = '<p>%s</p>' % buf
        return buf

    def print_rss(self):
        rss = RSS(encode = "UTF-8",
                  title = self.message["logo"],
                  parent = "http://" + self.environ["HTTP_HOST"],
                  uri = "http://" + self.environ["HTTP_HOST"]
                                  + self.gateway_cgi + self.sep + "rss",
                  description = self.message["description"],
                  xsl = config.xsl)
        cachelist = CacheList()
        now = int(time())
        for cache in cachelist:
            if cache.stamp + config.rss_range >= now:
                title = self.escape(self.file_decode(cache.datfile))
                path = self.appli[cache.type]+self.sep+self.str_encode(title)
                if cache.type == "thread":
                    last = cache.load_last()
                else:
                    try:
                        last = cache.load_last()[-1:]
                    except IndexError:
                        last = []
                for r in last:
                    desc = self.rss_text_format(r.get("body", ""))
                    content = self.rss_html_format(r.get("body", ""))
                    if cache.type == "thread":
                        permapath = "%s#r%s" % (path[1:], r.id[:8])
                    else:
                        permapath = path[1:]
                    rss.append(permapath,
                               date = r.stamp,
                               title = title,
                               description = desc,
                               content = content)

        self.stdout.write("Content-Type: text/xml; charset=UTF-8\n")
        try:
            self.stdout.write("Last-Modified: %s\n" %
                              self.rfc822_time(rss[rss.keys()[0]].date))
        except IndexError, KeyError:
            pass
        self.stdout.write("\n")
        if config.rss == "1":
            self.stdout.write(make_rss1(rss))
        else:
            self.stdout.write(make_rss2(rss))

    def print_motd(self):
        self.stdout.write("Content-Type: text/plain\n\n")
        try:
            f = file(config.motd)
            for line in f:
                self.stdout.write(line)
            f.close()
        except IOError:
            self.stderr.write(config.motd + ": IOError\n")

# End of CGI
